@@ -2,6 +2,7 @@ from email import header
 from pydoc import cli
 from tkinter import NS
 from typing import AsyncContextManager
+from typing_extensions import final
 from async_timeout import asyncio
 from diskcache import Cache
 from threading import Lock
@@ -176,7 +177,7 @@ class Client(Singleton):
         else:
             return False, '未受支持的Backend'
         
-    def put_from_cloud(self, url, bucket, bucket_name, timeout=10, file_ttl=-1):
+    def put_from_cloud(self, url, bucket, bucket_name, timeout=120, file_ttl=-1):
         # save to cache
         task_id = 'tmp_file_{}_{}'.format(bucket, bucket_name)
         try:
@@ -202,37 +203,38 @@ class Client(Singleton):
                         else:
                             return False, resp['err']
                     
-                    time.sleep(1)
+                    time.sleep(1/30)
             
             return False, '上传出错'
         except Exception as e:
             return False, str(e)
     
-    def put(self, path, bucket, bucket_name, timeout=10, file_ttl=-1):
+    def put(self, path, bucket, bucket_name, timeout=60, file_ttl=-1):
         ret, e = self.put_to_cloud(path, bucket, bucket_name)
         if ret:
             ret, e = self.put_from_cloud(e, bucket, bucket_name, timeout, file_ttl)
                 
         return ret, e
     
-    def get(self, path, url, force_copy=False, min_size=0, timeout=10):
-        if '://' in url:
-            ret, e = self.get_from_cloud(path, url, force_copy, min_size)
-            return ret, e
-        
-        print('downloading {}...'.format(url))
+    def get(self, path, url, force_copy=False, min_size=0, timeout=60, refresh=False):
         bucket, bucket_name = url.split('/')
         task_id = 'tmp_file_{}_{}'.format(bucket, bucket_name)
+        if not refresh:
+            if '://' in url:
+                ret, e = self.get_from_cloud(path, url, force_copy, min_size)
+                return ret, e
+            
+            print('downloading {}...'.format(url))
 
-        cached_resp = self.redis.get(task_id)
+            cached_resp = self.redis.get(task_id)
 
-        if cached_resp is not None and len(cached_resp) > 0:
-            resp = json.loads(cached_resp)
-            if resp['ret']:
-                tmp_url = resp['url']
-                return self.get_from_cloud(path, tmp_url, force_copy, min_size)
-
-        
+            if cached_resp is not None and len(cached_resp) > 0:
+                resp = json.loads(cached_resp)
+                if resp['ret']:
+                    tmp_url = resp['url']
+                    return self.get_from_cloud(path, tmp_url, force_copy, min_size)
+        else:
+            self.redis.delkey(task_id)
         ret = self.mq.publish(self.get_topic, {
             'bucket': bucket,
             'bucket_name': bucket_name
@@ -251,10 +253,59 @@ class Client(Singleton):
                         tmp_url = resp['url']
                         return self.get_from_cloud(path, tmp_url, force_copy, min_size)
                 
-                time.sleep(1)
+                time.sleep(1/30)
                 
         return False, '获取失败'
 
+    async def get_async(self, path, url, force_copy=False, min_size=0, timeout=60, refresh=False):
+        bucket, bucket_name = url.split('/')
+        task_id = 'tmp_file_{}_{}'.format(bucket, bucket_name)
+        if not refresh:
+            if '://' in url:
+                ret, e = self.get_from_cloud(path, url, force_copy, min_size)
+                return ret, e
+            
+            print('downloading {}...'.format(url))
+
+            cached_resp = self.redis.get(task_id)
+
+            if cached_resp is not None and len(cached_resp) > 0:
+                resp = json.loads(cached_resp)
+                if resp['ret']:
+                    tmp_url = resp['url']
+                    return self.get_from_cloud(path, tmp_url, force_copy, min_size)
+        else:
+            self.redis.delkey(task_id)
+        
+        connection = await self.mq.get_mq_conn()
+        try:
+            ret = await self.mq.async_publish(self.get_topic, {
+                'bucket': bucket,
+                'bucket_name': bucket_name,
+                'refresh': 'yes' if refresh else 'no'
+            }, mq=connection)
+            if ret:
+                await connection.close()
+                t = time.time()
+                resp = None
+                while True:
+                    if time.time() - t >= timeout:
+                        break
+                    
+                    resp = self.redis.get(task_id)
+                    if resp is not None and len(resp) > 0:
+                        resp = json.loads(resp)
+                        if resp['ret']:
+                            tmp_url = resp['url']
+                            return self.get_from_cloud(path, tmp_url, force_copy, min_size)
+                    
+                    await asyncio.sleep(1/30)
+        except Exception as e:
+            return False, '获取失败'
+        finally:
+            if not connection.is_closed:
+                await connection.close()
+
     async def get_mq_conn(self):
         return await self.mq.get_mq_conn()
 
@@ -283,7 +334,7 @@ class Client(Singleton):
                     else:
                         return False, resp['err']
                 
-                time.sleep(1)
+                await asyncio.sleep(1/30)
         return False, '上传出错'
     
     async def put_async(self, path, bucket, bucket_name, timeout=10, file_ttl=-1, mq=None):
@@ -292,44 +343,6 @@ class Client(Singleton):
             ret, e = await self.put_from_cloud_async(e, bucket, bucket_name, timeout, file_ttl)
         return ret, e
     
-    async def get_async(self, path, url, force_copy=False, min_size=0, timeout=10, mq=None):
-        if '://' in url:
-            ret, e = self.get_from_cloud(path, url, force_copy, min_size)
-            return ret, e
-        
-        bucket, bucket_name = url.split('/')
-        task_id = 'tmp_file_{}_{}'.format(bucket, bucket_name)
-
-        cached_resp = self.redis.get(task_id)
-
-        if cached_resp is not None and len(cached_resp) > 0:
-            resp = json.loads(cached_resp)
-            if resp['ret']:
-                tmp_url = resp['url']
-                return self.get_from_cloud(path, tmp_url, force_copy, min_size)
-        
-        ret = await self.mq.async_publish(self.get_topic, {
-            'bucket': bucket,
-            'bucket_name': bucket_name
-        }, mq=mq)
-        if ret:
-            t = time.time()
-            resp = None
-            while True:
-                if time.time() - t >= timeout:
-                    break
-                
-                resp = self.redis.get(task_id)
-                if resp is not None and len(resp) > 0:
-                    resp = json.loads(resp)
-                    if resp['ret']:
-                        tmp_url = resp['url']
-                        return self.get_from_cloud(path, tmp_url, force_copy, min_size)
-                
-                time.sleep(1)
-                
-        return False, '获取失败'
-    
     def geturl(self, entrypoint):
         prefix, bucket_info = entrypoint.split('://')
         if prefix in self.backend_pools:
